/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.sun.jsat;

import org.sun.jsat.controllers.FilterChangedController;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JTree;
import org.sun.jsat.controllers.DiffStartController;
import org.sun.jsat.controllers.InitializeAddController;
import org.sun.jsat.controllers.InitializeExportController;
import org.sun.jsat.controllers.InitializeImportController;
import org.sun.jsat.controllers.RemoveController;
import org.sun.jsat.controllers.StartupController;
import org.sun.jsat.controllers.StaticAnalysisController;
import org.sun.jsat.trees.CodeGroupingRootTreeNode;
import org.sun.jsat.trees.RootTreeNodeMaker;
import securityanalyzer.resourcemodel.FilledActionDefinition;
import securityanalyzer.resourcemodel.FilledRuleDefinition;
import securityanalyzer.useframework.SecurityModel;

/**
 *
 * @author joe
 */
public class JSATMain {

    SecurityModel sm;
    SecurityToolTopComponent mainWindow;
    String[] filterList;
    String currentPolicy;
    String defaultPolicy;
    RootTreeNodeMaker rootNodes;

    public void init() {

        mainWindow = SecurityToolTopComponent.findInstance();



        sm = new SecurityModel();

        rootNodes = new RootTreeNodeMaker(sm);

        currentPolicy = "";
        defaultPolicy = "";

        StartupController startup = new StartupController(this);
        startup.doStartup();

        InitializeAddController actionAdder = new InitializeAddController(this, AddDialog.ADDING_ACTIONS);
        mainWindow.getAddActionButton().addActionListener(actionAdder);

        InitializeAddController ruleAdder = new InitializeAddController(this, AddDialog.ADDING_RULES);
        mainWindow.getAddRuleButton().addActionListener(ruleAdder);

        JTree actionTree = mainWindow.getActionTree();
        RemoveController removeActionController = new RemoveController(this, actionTree);
        mainWindow.getRemoveActionButton().addActionListener(removeActionController);

        JTree ruleTree = mainWindow.getRuleTree();
        RemoveController removeRuleController = new RemoveController(this, ruleTree);
        mainWindow.getRemoveRuleButton().addActionListener(removeRuleController);

        JButton importButton = mainWindow.getImportButton();
        InitializeImportController iic = new InitializeImportController(this);
        importButton.addActionListener(iic);

        JButton exportButton = mainWindow.getExportButton();
        InitializeExportController iec = new InitializeExportController(this);
        exportButton.addActionListener(iec);

        JComboBox filterBox = mainWindow.getFilterSelectionComboBox();
        FilterChangedController fcc = new FilterChangedController(this, filterBox);
        filterBox.addActionListener(fcc);

        DiffStartController dsc = new DiffStartController(this);
        mainWindow.getDiffButton().addActionListener(dsc);

        mainWindow.getStaticAnalysisButton().addActionListener(new StaticAnalysisController(this));
    }

    public void initCodeGroup() {

        mainWindow.setCodeGrouping(sm.getDefaultCodeGroup());
    }

    public void addAction(FilledActionDefinition fad) {
        sm.getDefaultCodeGroup().getExternalPolicyByName(currentPolicy).addAction(fad);
        mainWindow.refreshTrees();
    }

    public void addRule(FilledRuleDefinition frd) {
        sm.getDefaultCodeGroup().getExternalPolicyByName(currentPolicy).addRule(frd);
        mainWindow.refreshTrees();
    }

    public void removeAction(FilledActionDefinition fad) {
        sm.getDefaultCodeGroup().getExternalPolicyByName(currentPolicy).removeAction(fad);
        mainWindow.refreshTrees();
    }

    public void removeRule(FilledRuleDefinition frd) {

        sm.getDefaultCodeGroup().getExternalPolicyByName(currentPolicy).removeRule(frd);

        mainWindow.refreshTrees();
    }

    public SecurityModel getSecurityModel() {
        return sm;
    }

    public SecurityToolTopComponent getMainWindow() {
        return mainWindow;
    }

    public void refreshWindow() {
        mainWindow.refreshTrees();
    }

    public void changeTreeRoots(CodeGroupingRootTreeNode actionRoot,
            CodeGroupingRootTreeNode ruleRoot) {
        mainWindow.setRootRuleNode(ruleRoot);
        mainWindow.setRootActionNode(actionRoot);
        refreshWindow();
    }

    public void setCurrentPolicyName(String name) {
        currentPolicy = name;
    }

    public void setDefaultPolicyName(String name) {
        defaultPolicy = name;
    }

    public String getDefaultPolicyName() {
        return defaultPolicy;
    }
    
    public String getCurrentPolicyName() {
        return currentPolicy;
    }

    public RootTreeNodeMaker getRootNodes() {
        return rootNodes;
    }

    public void refreshFilterList() {

        filterList = getAllPolicyNames();

        mainWindow.getFilterSelectionComboBox().setModel(new DefaultComboBoxModel(filterList));
        mainWindow.getFilterSelectionComboBox().setSelectedItem(currentPolicy);

    }
    
    public String[] getAllPolicyNames() {
                List<String> filters = new ArrayList<String>();

        Collection<String> externals = sm.getDefaultCodeGroup().getAllExternalPolicyNames();
        Collection<String> analyses = sm.getDefaultCodeGroup().getAllAnalysisPolicyNames();

        for (String s : externals) {
            filters.add(s);
        }

        for (String s : analyses) {
            filters.add(s);
        }

        return filters.toArray(new String[0]);
    }
}
