package securityanalyzer.analysismodel;

import java.util.Collection;
import java.util.Map;
import java.util.TreeMap;

/**
 * 
 * Describes the top level container for storing analysis results
 * 
 */
public class AnalysisModel {

	/** Contains the singleton instance */
	private static AnalysisModel instance = new AnalysisModel();

	/** Contains all results */
	private static Map<String,CodeBlock> results = new TreeMap<String,CodeBlock>();

	/** Constructor should never be called by anyone */
	private AnalysisModel() {
	}

	/**
	 * get the singleton instance of the AnalysisModel
	 * 
	 * @return the singleton instance of the AnalysisModel
	 */
	public static AnalysisModel getInstance() {
		return instance;
	}

	/**
	 * get the list of all results in the AnalysisModel
	 * 
	 * @return list of all results in the AnalysisModel
	 */
	public static Collection<CodeBlock> getAllCodeBlocks() {
		return results.values();
	}

	/**
	 * add a {@link CodeBlock} of the specified type and name
	 * 
	 * @param type
	 * @param name
	 * @return the newly created {@link CodeBlock}
	 */
	public static CodeBlock addCodeBlock(String type, String name) {
		// TODO check for duplicates?
		CodeBlock cb = findByName(name);
		
		if(cb == null) {
			cb = new CodeBlock(type, name);
			results.put(name,cb);
		}
		else {
			if (!cb.getType().equalsIgnoreCase(type)) {
				throw new RuntimeException("A CodeBlock with name " + name
						+ " exists, but it has type " + cb.getType() + ", and "
						+ type + " was specified");
			}
		}
		
		
		return cb;
	}

	public static Method createMethod(String fullyQualifiedName,
			CodeBlock parentClass) {

		CodeBlock cb = findByName(fullyQualifiedName);

		if (cb != null) {
			if (cb instanceof Method) {
				if (!cb.getParent().equals(parentClass)) {
					throw new RuntimeException(
							"An object with fullyQualifiedName "
									+ fullyQualifiedName
									+ " exists, but parent is "
									+ cb.getParent().getFullyQualifiedName()
									+ " and specified parent is "
									+ parentClass.getFullyQualifiedName());
				}

				return (Method) cb;
				
			} else {
				throw new ClassCastException(
						"An object with fullyQualifiedName "
								+ fullyQualifiedName
								+ " exists, but it is not this method.");
			}
		}

		cb = new Method(fullyQualifiedName, parentClass);
		results.put(fullyQualifiedName, cb);
		
		return (Method) cb;

	}

	public static CodeBlock findByName(String fullyQualifiedName) {
		return results.get(fullyQualifiedName);
	}
}
