package securityanalyzer.analysismodel;

import java.util.ArrayList;
import java.util.List;

/**
 * 
 * Describes a Java method in a very simple way
 *
 */
public class Method extends CodeBlock {
	
	/** List of all MethodInvocations that could be caused by this Method */
	private List<MethodInvocation> invocationsOut;
	
	/** List of all MethodInvocations that could invoke this Method */
	private List<MethodInvocation> invocationsIn;
	
	/**
	 * Constructor
	 * @param fullyQualifiedName The FQN of this Method
	 * @param parentClass The parent class of this method
	 */
	public Method(String fullyQualifiedName, CodeBlock parentClass) {
		super("method", fullyQualifiedName);
		this.setParent(parentClass);
		this.invocationsOut = new ArrayList<MethodInvocation>();
		this.invocationsIn = new ArrayList<MethodInvocation>();
	}
	
	public boolean isAppCode() {
		return getParent().isAppCode();
	}
	
	/**
	 * Get all MethodInvocations that could be caused by this Method
	 * @return
	 */
	public List<MethodInvocation> getAllMethodInvocationsOut(){
		return invocationsOut;
	}
	
	/**
	 * Add a MethodInvocation to the list of possible invocations
	 * @param i
	 */
	public void addMethodInvocationOut(MethodInvocation i){
		invocationsOut.add(i);
	}
	
	/**
	 * Remove a MethodInvocation from the list of possible invocations
	 * @param i
	 * @return
	 */
	public boolean removeMethodInvocationOut(MethodInvocation i){
		return invocationsOut.remove(i);
	}
	
	/**
	 * Get all MethodInvocations that could come into this Method
	 * @return
	 */
	public List<MethodInvocation> getAllMethodInvocationsIn(){
		return invocationsIn;
	}
	
	/**
	 * Add a MethodInvocation to the list of incoming invocations
	 * @param i
	 */
	public void addMethodInvocationIn(MethodInvocation i){
		invocationsIn.add(i);
	}
	
	/**
	 * Remove a MethodInvocation from the list of incoming invocations
	 * @param i
	 * @return
	 */
	public boolean removeMethodInvocationIn(MethodInvocation i){
		return invocationsIn.remove(i);
	}
	

}
