package securityanalyzer.resourcemodel;

import java.util.HashMap;
import java.util.Map;

public class ResourceModelAssociationMap<T> {

	private Map<T, RulesAndActions> map;
	
	public ResourceModelAssociationMap() {
		map = new HashMap<T, RulesAndActions>();
	}
	
	public RulesAndActions get(T key){
		return map.get(key);
	}
	
	public RulesAndActions getOrAdd(T key){
		RulesAndActions ret = get(key);
		if (ret == null){
			ret = new RulesAndActions();
			map.put(key, ret);
		}
		return ret;		
	}
	
	public boolean add(T key, FilledRuleDefinition rule){
		RulesAndActions raa = getOrAdd(key);
		return raa.rules.add(rule);
	}
	
	public boolean add(T key, FilledActionDefinition rule){
		RulesAndActions raa = getOrAdd(key);
		return raa.actions.add(rule);
	}
	
	public RulesAndActions removeAll(T key){
		return map.remove(key);
	}
	
	public boolean removeRule(T key, FilledRuleDefinition rule){
		RulesAndActions raa = get(key);
		if (raa == null) return false;
		return raa.rules.remove(rule);
	}
	
	public boolean removeAction(T key, FilledActionDefinition action){
		RulesAndActions raa = get(key);
		if (raa == null) return false;
		return raa.actions.remove(action);
	}
	
	public FilledActionDefinitionCollection getActions(T key){
		RulesAndActions raa = get(key);
		if (raa == null) return null;
		return raa.actions;
	}
	
	public FilledRuleDefinitionCollection getRules(T key){
		RulesAndActions raa = get(key);
		if (raa == null) return null;
		return raa.rules;
	}
	
	public FilledRuleDefinitionCollection getRules(T key, String ruleType){
		FilledRuleDefinitionCollection all = getRules(key);
		if (all == null) return null;
		FilledRuleDefinitionCollection ret = new FilledRuleDefinitionCollection();
		for (FilledRuleDefinition frd : all){
			//TODO is this getType call correct?
			if (frd.getBaseObject().getType().equals(ruleType)) {
				ret.add(frd);
			}
		}
		return ret;
	}
	
	protected static class RulesAndActions {
		public FilledActionDefinitionCollection actions;
		
		public FilledRuleDefinitionCollection rules;
		
		public RulesAndActions() {
			actions = new FilledActionDefinitionCollection();
			rules = new FilledRuleDefinitionCollection();
		}
	}
}
