/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package securityanalyzer.resourcemodel.schemamodel;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import securityanalyzer.resourcemodel.ActionDefinition;
import securityanalyzer.resourcemodel.ActionDefinition.AppliedRule;
import securityanalyzer.resourcemodel.ParameterDefinition;
import securityanalyzer.resourcemodel.ResourceDefinition;
import securityanalyzer.resourcemodel.ResourceModel;
import securityanalyzer.resourcemodel.RuleDefinition;

/**
 *
 * @author joe
 */
public class ResourceModelWriter {

    public static void saveResourcesAndRules(File f) {
        
        ResourceToRulesType rootElement = new ResourceToRulesType();
        
        rootElement.resource = getAndPopulateResources();
        //rootElement.rule = getAndPopulateRules();
        
    }
    
    public static List<ResourceType> getAndPopulateResources() {
        
        Set<ResourceDefinition> rds = ResourceModel.getAllResources();
        List<ResourceType> ret = new ArrayList<ResourceType>();
        
        
        for(ResourceDefinition rd: rds) {
            ResourceType rt = populateResource(rd);
            ret.add(rt);
        }
        
        return ret;
        
    }
    
    public static ResourceType populateResource(ResourceDefinition rd) {
        
        ResourceType rt = new ResourceType();
        rt.description = rd.getDescription();
        rt.name = rd.getName();
        
        rt.action = populateActions(rd.getActions());
        
        return rt;
        
    }
    
    public static List<ActionType> populateActions(Set<ActionDefinition> ads) {
        
        List<ActionType> ret = new ArrayList<ActionType>();
        
        for(ActionDefinition ad: ads) {
            ActionType at = populateAction(ad);
            ret.add(at);
        }
        
        return ret;
    }
    
    public static ActionType populateAction(ActionDefinition ad) {
        
        ActionType at = new ActionType();
        at.name = ad.getName();
        at.description = ad.getDescription();
        
        at.actionParameters = populateActionParameters(ad.getParameters());
        at.appliedRules = populateAppliedRules(ad.getParameterApplications(RuleDefinition.TYPE_PERMISSION));
        
        return at;
    }
    
    public static ActionParametersType populateActionParameters(List<ParameterDefinition> pds) {
        
        ActionParametersType ret = new ActionParametersType();
        List<ActionParameterType> list = ret.getActionParameter();
        
        for(ParameterDefinition pd: pds) {
            ActionParameterType apt = populateActionParameter(pd);
            list.add(apt);
            
        }
        
        return ret;
    }
    
    public static ActionParameterType populateActionParameter(ParameterDefinition pd) {
        
        ActionParameterType apt = new ActionParameterType();
        
        apt.description = pd.getDescription();
        apt.name = pd.getName();
        
        return apt;
        
    }
    
    public static AppliedRulesType populateAppliedRules(Set<AppliedRule> ars) {
        
        AppliedRulesType ret = new AppliedRulesType();
        List<AppliedRuleType> list = ret.getAppliedRule();
        
        for(AppliedRule ar: ars) {
            AppliedRuleType art = populateAppliedRule(ar);
            list.add(art);
        }
        
        return ret;
        
    }
    
    public static AppliedRuleType populateAppliedRule(AppliedRule ar) {
        
        AppliedRuleType art = new AppliedRuleType();
        
        art.name = ar.ruleDefinition.getName();
        art.type = ar.ruleDefinition.getType();
        
        art.parameterApplications = populateParameterApplications(ar.nameValueMap);
        
        return art;
        
    }
    
    public static ParameterApplicationsType populateParameterApplications(Map<String,String> apps) {
        
        ParameterApplicationsType ret = new ParameterApplicationsType();
        List<ParameterApplicationType> list = ret.getParameterApplication();
        
        int i=0;
        
        for(Entry<String, String> e: apps.entrySet()) {
            
            ParameterApplicationType pat = new ParameterApplicationType();
            pat.id = Integer.toString(i);
            i++;
            
            pat.name = e.getKey();
            pat.value = e.getValue();
        }
        
        return ret;
        
    }
    
}
