/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package securityanalyzer.useframework;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import securityanalyzer.resourcemodel.FilledRuleDefinition;
import securityanalyzer.resourcemodel.ParameterDefinition;
import securityanalyzer.resourcemodel.ResourceModel;
import securityanalyzer.resourcemodel.RuleDefinition;

/**
 *
 * @author joe
 */
public class PolicyConfigurationReader implements PolicyImporter {
    
    private File policyFile;
    
    public PolicyConfigurationReader(String filename){
        this.policyFile = new File(filename);
    }
    
    public PolicyConfigurationReader(File file){
        this.policyFile = file;
    }

    public static List<FilledRuleDefinition> getRulesFromFile(String filename) {
        return getRulesFromFile(new File(filename));
    }

    public static List<FilledRuleDefinition> getRulesFromFile(File file) {
        List<FilledRuleDefinition> frds = new ArrayList<FilledRuleDefinition>();

        try {
            InputStream fileReader = new FileInputStream(file);
            BufferedReader reader = new BufferedReader(new InputStreamReader(fileReader));
            frds = getRulesFromBufferedReader(reader);

        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }


        return frds;
    }

    public static List<FilledRuleDefinition> getRulesFromBufferedReader(BufferedReader reader) {



        String s = "";
        // our eventual return value
        List<FilledRuleDefinition> frds = new ArrayList<FilledRuleDefinition>();

        while (true) {

            try {
                s = reader.readLine();

                // reached the end of the file, all done
                if (s == null) {
                    break;
                }

                // split on open curly braces and space
                String[] tokens = s.split(" \\{");

                // first thing is classname
                String classname = tokens[0];

                // the rest is all arguments
                String[] args = new String[tokens.length - 1];

                // Remove ending curly brace and feed into args
                for (int j = 1; j < tokens.length; j++) {
                    args[j - 1] = tokens[j].substring(0, tokens[j].length() - 1);
                }

                // Get short classname from the end of the fqn, use to get 
                // ruledef and make filledruledef
                String name = classname.substring(classname.lastIndexOf('.') + 1);
                RuleDefinition rd = ResourceModel.getRuleByName(name);
                FilledRuleDefinition frd = new FilledRuleDefinition(rd);

                // get parameters and make sure input size matches 
                List<ParameterDefinition> pds = rd.getParameters();

                if (pds.size() != args.length) {
                    System.err.println("Incorrect # of args in permission definition" + name);
                    return null;
                }

                // copy over input parameters into mappings in the frd
                int i = 0;
                for (ParameterDefinition pd : pds) {
                    frd.setMapping(pd, args[i++]);
                }

                // finally, add the frd to the list
                frds.add(frd);

            } catch (IOException e) {
                e.printStackTrace();
                return null;
            }
        }

        // return the complete list of frds
        return frds;


    }

    public void importPolicy(RulesActionsContainer rac) {
        rac.addRules(getRulesFromFile(policyFile));
        rac.updateActionsFromRules();
    }
}
