/**
 * 
 */
package securityanalyzer.useframework;

import java.util.Collection;
import java.util.LinkedList;

import securityanalyzer.resourcemodel.FilledActionDefinition;
import securityanalyzer.resourcemodel.FilledActionDefinitionCollection;
import securityanalyzer.resourcemodel.FilledRuleDefinition;
import securityanalyzer.resourcemodel.FilledRuleDefinitionCollection;
import securityanalyzer.resourcemodel.RuleDefinition;

public class RulesActionsContainer {

    private FilledActionDefinitionCollection actions = new FilledActionDefinitionCollection();
    private FilledRuleDefinitionCollection rules = new FilledRuleDefinitionCollection();
    private Collection<FilledRuleDefinition> unmatchedRules = new LinkedList<FilledRuleDefinition>();

    public void updateActionsFromRules() {
        //TODO - this should be more flexible, not just for permission rules!
        FilledActionDefinitionCollection newActions = rules.resolveToActions(RuleDefinition.TYPE_PERMISSION);
        unmatchedRules.addAll(rules.getUnused());
        
        
        //Add new actions
        for (FilledActionDefinition frd : newActions) {
            if (!actions.contains(frd)) {
                actions.add(frd);
            }
        }

        //Remove old actions
        Collection<FilledActionDefinition> removeActions = new LinkedList<FilledActionDefinition>();
        for (FilledActionDefinition frd : actions) {
            if (!newActions.contains(frd)) {
                removeActions.add(frd);
            }
        }
        actions.removeAll(removeActions);

        updateUnmatched();
    }

    public void updateRulesFromActions() {
        updateUnmatched();

        //TODO - this should be more flexible, not just for permission rules!
        FilledRuleDefinitionCollection newRules = actions.resolveToRules(RuleDefinition.TYPE_PERMISSION);

        for (FilledRuleDefinition frd : newRules) {
            if (!rules.contains(frd)) {
                rules.add(frd);
            }
        }

        Collection<FilledRuleDefinition> removeRules = new LinkedList<FilledRuleDefinition>();
        for (FilledRuleDefinition frd : rules) {
            if (!newRules.contains(frd) && !unmatchedRules.contains(frd)) {
                removeRules.add(frd);
            }
        }
        rules.removeAll(removeRules);
    }

    private void updateUnmatched() {
        //Update list of used rules
        Collection<FilledRuleDefinition> newlyMatchedRules = new LinkedList<FilledRuleDefinition>();
        for (FilledRuleDefinition frd : unmatchedRules) {
            for (FilledActionDefinition fad : actions) {
                //TODO - this should be more flexible, not just for permission rules!
                if (fad.resolveToRuleDefinitions(RuleDefinition.TYPE_PERMISSION).contains(frd)) {
                    newlyMatchedRules.add(frd);
                }
            }
        }
        unmatchedRules.removeAll(newlyMatchedRules);
    }

    public FilledActionDefinitionCollection getActions() {
        return actions;
    }

    public FilledRuleDefinitionCollection getRules() {
        return rules;
    }

    public Collection<FilledRuleDefinition> getUnmatchedRules() {
        return unmatchedRules;
    }

    public boolean isUnmatched(FilledRuleDefinition frd) {
        return unmatchedRules.contains(frd);
    }

    public boolean addActions(Collection<FilledActionDefinition> fads) {
        boolean ret = actions.addAll(fads);
        updateRulesFromActions();
        updateActionsFromRules();
        return ret;
    }

    public boolean addAction(FilledActionDefinition fad) {
        boolean ret = actions.add(fad);
        updateRulesFromActions();
        updateActionsFromRules();
        return ret;
    }

    public boolean addRules(Collection<FilledRuleDefinition> frds) {
        boolean ret = rules.addAll(frds);
        updateActionsFromRules();
        return ret;
    }

    public boolean addRule(FilledRuleDefinition frd) {
        boolean ret = rules.add(frd);
        updateActionsFromRules();
        return ret;
    }

    public boolean removeAction(FilledActionDefinition fad) {
        boolean ret = actions.remove(fad);
        updateRulesFromActions();
        return ret;
    }

    public boolean removeRule(FilledRuleDefinition frd) {
        boolean ret = rules.remove(frd);
        updateActionsFromRules();
        return ret;
    }
}