package securityanalyzer.resourcemodel;

import java.io.File;
import java.util.List;
import java.util.Map.Entry;

import org.junit.Test;

import securityanalyzer.resourcemodel.reader.ResourceModelReader;

public class FilledRuleDefinitionCollectionTest {

	public void startUp() {
		
	}
	
	@Test
	public void testResolveToActionsWithFiles() {
		
		//Need to get some rule definitions, easiest to
		//do by reading in from xml
		ResourceModelReader.setResourcesAndPermissions(new File(
		"Resource Model/secondDraft.xml"));
		
		RuleDefinition rd = ResourceModel.getRuleByName("FilePermission");
		
		FilledRuleDefinition frd1 = new FilledRuleDefinition(rd);
		
		List<ParameterDefinition> pds = rd.getParameters();
		
		frd1.setMapping(pds.get(0), "/tmp/abc/-");
		frd1.setMapping(pds.get(1), "read");
		
		FilledRuleDefinition frd2 = new FilledRuleDefinition(rd);
		
		frd2.setMapping(pds.get(0), "/tmp/ibc/-");
		frd2.setMapping(pds.get(1), "execute");
		
		FilledRuleDefinitionCollection frds = new FilledRuleDefinitionCollection();
		
		frds.add(frd1);
		frds.add(frd2);
		
		System.out.println("Original FilledRuleDefinitions:");
		printFRDC(frds);
		
		FilledActionDefinitionCollection fads = frds.resolveToActions("Permission");
		
		System.out.println("\nConverted to Actions:");
		printFADC(fads);
		
		List<FilledRuleDefinition> unmatchedRules = frds.getUnused();
		
		System.out.println("\nFound " + unmatchedRules.size() + " unmatched rule(s):");
		printFRDC(unmatchedRules);
		
		frds.clear();
		System.out.println("\nAll actions converted back to rules:");
		for(FilledActionDefinition fad: fads) {
			frds.addAll(VariableResolution.getFilledRulesFrom(fad, "Permission"));
		}
		
		printFRDC(frds);
		
		cleanUp();
		
		return;
	}
	
	@Test
	public void testResolveToActionsWithMixedRules() {
		
		//Need to get some rule definitions, easiest to
		//do by reading in from xml
		ResourceModelReader.setResourcesAndPermissions(new File(
		"Resource Model/secondDraft.xml"));
		
		RuleDefinition rd = ResourceModel.getRuleByName("FilePermission");
		
		FilledRuleDefinition frd1 = new FilledRuleDefinition(rd);
		
		List<ParameterDefinition> pds = rd.getParameters();
		
		frd1.setMapping(pds.get(0), "/tmp/abc/-");
		frd1.setMapping(pds.get(1), "read");
		
		FilledRuleDefinition frd2 = new FilledRuleDefinition(rd);
		
		frd2.setMapping(pds.get(0), "/tmp/ibc/-");
		frd2.setMapping(pds.get(1), "execute");
		
		RuleDefinition rd2 = ResourceModel.getRuleByName("SocketPermission");
		
		FilledRuleDefinition frd3 = new FilledRuleDefinition(rd2);
		
		pds = rd2.getParameters();
		
		frd3.setMapping(pds.get(0), "*.sun.com:[8000:8200]");
		frd3.setMapping(pds.get(1), "connect");
		
		FilledRuleDefinitionCollection frds = new FilledRuleDefinitionCollection();
		
		frds.add(frd1);
		frds.add(frd2);
		frds.add(frd3);
		
		System.out.println("Original FilledRuleDefinitions:");
		printFRDC(frds);
		
		FilledActionDefinitionCollection fads = frds.resolveToActions("Permission");
		
		System.out.println("\nConverted to Actions:");
		printFADC(fads);
		
		List<FilledRuleDefinition> unmatchedRules = frds.getUnused();
		
		System.out.println("\nFound " + unmatchedRules.size() + " unmatched rule(s):");
		printFRDC(unmatchedRules);
		
		frds.clear();
		System.out.println("\nAll actions converted back to rules:");
		for(FilledActionDefinition fad: fads) {
			frds.addAll(VariableResolution.getFilledRulesFrom(fad, "Permission"));
		}
		
		printFRDC(frds);
		
		cleanUp();
		
		return;
	}
	
	@Test
	public void testResolveToActionsWithSpecificSet() {
		
		//Need to get some rule definitions, easiest to
		//do by reading in from xml
		ResourceModelReader.setResourcesAndPermissions(new File(
		"Resource Model/secondDraft.xml"));
		
		RuleDefinition rd = ResourceModel.getRuleByName("RuntimePermission");
		
		FilledRuleDefinition frd1 = new FilledRuleDefinition(rd);
		List<ParameterDefinition> pds = rd.getParameters();
		frd1.setMapping(pds.get(0), "getClassLoader");
		
		FilledRuleDefinition frd2 = new FilledRuleDefinition(rd);
		frd2.setMapping(pds.get(0), "createClassLoader");
		
		FilledRuleDefinition frd3 = new FilledRuleDefinition(rd);
		frd3.setMapping(pds.get(0), "setContextClassLoader");
		
		FilledRuleDefinition frd4 = new FilledRuleDefinition(rd);
		frd4.setMapping(pds.get(0), "enableContextClassLoaderOverride");
		
		FilledRuleDefinitionCollection frds = new FilledRuleDefinitionCollection();
		
		frds.add(frd1);
		frds.add(frd2);
		frds.add(frd3);
		frds.add(frd4);
		
		System.out.println("Original FilledRuleDefinitions:");
		printFRDC(frds);
		
		FilledActionDefinitionCollection fads = frds.resolveToActions("Permission");
		
		System.out.println("\nConverted to Actions:");
		printFADC(fads);
		
		List<FilledRuleDefinition> unmatchedRules = frds.getUnused();
		
		System.out.println("\nFound " + unmatchedRules.size() + " unmatched rule(s):");
		printFRDC(unmatchedRules);
		
		frds.clear();
		System.out.println("\nAll actions converted back to rules:");
		for(FilledActionDefinition fad: fads) {
			frds.addAll(VariableResolution.getFilledRulesFrom(fad, "Permission"));
		}
		
		printFRDC(frds);
		
		cleanUp();
		
		return;
	}
	
	@Test
	public void severalSetsTest() {
		//Need to get some rule definitions, easiest to
		//do by reading in from xml
		ResourceModelReader.setResourcesAndPermissions(new File(
		"Resource Model/secondDraft.xml"));
		
		RuleDefinition rd = ResourceModel.getRuleByName("RuntimePermission");
		
		FilledRuleDefinition frd1 = new FilledRuleDefinition(rd);
		List<ParameterDefinition> pds = rd.getParameters();
		frd1.setMapping(pds.get(0), "getClassLoader");
		
		FilledRuleDefinition frd2 = new FilledRuleDefinition(rd);
		frd2.setMapping(pds.get(0), "createClassLoader");
		
		FilledRuleDefinition frd3 = new FilledRuleDefinition(rd);
		frd3.setMapping(pds.get(0), "setContextClassLoader");
		
		FilledRuleDefinition frd4 = new FilledRuleDefinition(rd);
		frd4.setMapping(pds.get(0), "enableContextClassLoaderOverride");
		
		FilledRuleDefinition frd9 = new FilledRuleDefinition(rd);
		frd9.setMapping(pds.get(0), "exitVM");
		
		FilledRuleDefinition frd10 = new FilledRuleDefinition(rd);
		frd10.setMapping(pds.get(0), "shutdownHooks");
		
		FilledRuleDefinition frd11 = new FilledRuleDefinition(rd);
		frd11.setMapping(pds.get(0), "notMatchingAnything");
		
		FilledRuleDefinition frd5 = new FilledRuleDefinition(rd);
		frd5.setMapping(pds.get(0), "loadLibrary.externlib.*");
		
		RuleDefinition rd2 = ResourceModel.getRuleByName("FilePermission");
		
		List<ParameterDefinition> pds2 = rd2.getParameters();
		
		FilledRuleDefinition frd6 = new FilledRuleDefinition(rd2);
		frd6.setMapping(pds2.get(0), "/tmp/folder/*");
		frd6.setMapping(pds2.get(1), "read");
		
		FilledRuleDefinition frd7 = new FilledRuleDefinition(rd2);
		frd7.setMapping(pds2.get(0), "/tmp/otherfolder/*");
		frd7.setMapping(pds2.get(1), "write");
		
		FilledRuleDefinition frd8 = new FilledRuleDefinition(rd2);
		frd8.setMapping(pds2.get(0), "/tmp/otherfolder/*");
		frd8.setMapping(pds2.get(1), "read");
		
		RuleDefinition rd3 = ResourceModel.getRuleByName("AWTPermission");
		List<ParameterDefinition> pds3 = rd3.getParameters();
		
		FilledRuleDefinition frd12 = new FilledRuleDefinition(rd3);
		frd12.setMapping(pds3.get(0), "watchMousePointer");
		
		FilledRuleDefinition frd13 = new FilledRuleDefinition(rd3);
		frd13.setMapping(pds3.get(0), "accessEventQueue");
		
		FilledRuleDefinitionCollection frds = new FilledRuleDefinitionCollection();
		
		frds.add(frd1);
		frds.add(frd2);
		frds.add(frd3);
		frds.add(frd4);
		frds.add(frd5);
		frds.add(frd6);
		frds.add(frd7);
		frds.add(frd8);
		frds.add(frd9);
		frds.add(frd10);
		frds.add(frd11);
		frds.add(frd12);
		frds.add(frd13);
		
		System.out.println("Original FilledRuleDefinitions:");
		printFRDC(frds);
		
		FilledActionDefinitionCollection fads = frds.resolveToActions("Permission");
		
		System.out.println("\nConverted to Actions:");
		printFADC(fads);
		
		List<FilledRuleDefinition> unmatchedRules = frds.getUnused();
		
		System.out.println("\nFound " + unmatchedRules.size() + " unmatched rule(s):");
		printFRDC(unmatchedRules);
		
		frds.clear();
		System.out.println("\nAll actions converted back to rules:");
		for(FilledActionDefinition fad: fads) {
			frds.addAll(VariableResolution.getFilledRulesFrom(fad, "Permission"));
		}
		
		printFRDC(frds);

		cleanUp();
		
		return;	
	}
	
	public void printFRDC(Iterable<FilledRuleDefinition> frds) {
		for(FilledRuleDefinition frd: frds) {
			System.out.println("  " + frd.getBaseObject().getName());
			for(Entry<ParameterDefinition, String> e : frd.getAllDefinedMappings().entrySet()) {
				System.out.println("    " + e.getKey().getName() + "=" + e.getValue());
			}
		}
	}
	
	public void printFADC(FilledActionDefinitionCollection fads) {
		for(FilledActionDefinition fad: fads) {
			System.out.println("  " + fad.getBaseObject().getName());
			for(Entry<ParameterDefinition, String> e : fad.getAllDefinedMappings().entrySet()) {
				System.out.println("    " + e.getKey().getName() + "=" + e.getValue());
			}
		}
	}
	
	public void cleanUp() {
		ResourceModel.getAllResources().clear();
		ResourceModel.getAllRules().clear();
	}
}
